@@ -4,19 +4,35 @@ ext_Experiment is the container for whole project data.
 Heavy lifting is in constructors.CompositeMap, 
 which contains MassGrid for correspondence, and FeatureList from feature/peak detection.
 
+Annotation is facilitated by jms-metabolite-services, mass2chem
+
+from mass2chem.epdsConstructor import epdsConstructor
+
 '''
 import os
 import random
 import json
+import pickle
 
 from metDataModel.core import Experiment
 from mass2chem.search import *
-from mass2chem.epdsConstructor import epdsConstructor
+
+# jms-metabolite-services
+from jms.dbStructures import knownCompoundDatabase, ExperimentalEcpdDatabase
 
 from .samples import SimpleSample
 from .constructors import CompositeMap
+from .json_encoder import NpEncoder
 from .sql import *
 
+try:
+    import importlib.resources as pkg_resources
+except ImportError:
+    import importlib_resources as pkg_resources
+
+from . import db
+
+
 # General data processing steps are in this class
 class ext_Experiment(Experiment):
     '''
@@ -60,10 +76,6 @@ class ext_Experiment(Experiment):
         initiation_Samples are used to select one most representative sample to seed MassGrid and RT alignment.
         If refDB is used, it's better to be used after all samples are processed, 
         because the m/z values of samples are closer to other samples than refDB.
-        
-        if refDB:
-            self.CMAP.align_to_refdb(refDB)
-
         '''
         # start SQLite database
         # self.cursor = connect_sqlite_db(self.parameters['project_name'])
@@ -73,11 +85,12 @@ class ext_Experiment(Experiment):
         #      start DB after init
         
         self.process_all_without_export()
-
         # 
         self.CMAP.MassGrid.to_csv(
             os.path.join(self.parameters['outdir'], self.parameters['mass_grid_mapping']) )
+
         self.annotate()
+
         self.export_feature_table()
 
 
@@ -128,47 +141,125 @@ class ext_Experiment(Experiment):
 
 
     def annotate(self):
-        '''Will add DB match too
-        
         '''
-        outfile = os.path.join(self.parameters['outdir'], self.parameters['json_empricalCompounds'])
-        ECCON = epdsConstructor(self.CMAP.FeatureList, mode=self.mode)
-        list_empCpds = ECCON.peaks_to_epds()
+        To-do: Consider mass calibration before DB matching -
+            to add self.parameters
 
-        list_empCpds = self._reformat_epds_(list_empCpds, self.CMAP.FeatureList)
+        '''
+        self.load_annotation_db()
+        self.db_mass_calibrate()
+
+        EED = ExperimentalEcpdDatabase(mode=self.mode)
+        EED.build_from_list_peaks(self.CMAP.FeatureList)
+        EED.extend_empCpd_annotation(self.KCD)
+        EED.annotate_singletons(self.KCD)
+
+        self.export_peak_annotation(EED.dict_empCpds, self.KCD, 'peak_annotation')
+        
+        # also exporting JSON
+        outfile = os.path.join(self.parameters['outdir'], 'annotated_empricalCompounds.json')
         with open(outfile, 'w', encoding='utf-8') as f:
-            json.dump(list_empCpds, f, ensure_ascii=False, indent=2)
-
-        print("\nEmpirical compound annotaion (%d) was written to %s." %(len(list_empCpds), outfile))
+            json.dump(EED.dict_empCpds, f, cls=NpEncoder, ensure_ascii=False, indent=2)
 
 
-    def _reformat_epds_(self, list_empCpds, FeatureList):
-        fDict = {}
-        for F in FeatureList:
-            fDict[F['id_number']] = F
-        new = []
-        for E in list_empCpds:
-            features = []
-            for peak in E['list_peaks']:
-                features.append(
-                    {'feature_id': peak[0], 
-                    'mz': fDict[peak[0]]['mz'], 
-                    'rtime': fDict[peak[0]]['apex'], 
-                    'charged_formula': '', 
-                    'ion_relation': peak[1]}
-                )
-            new.append(
-                {
-                'interim_id': E['id'], 
-                'neutral_formula_mass': None,
-                'neutral_formula': None,
-                'Database_referred': [],
-                'identity': [],
-                'MS1_pseudo_Spectra': features,
-                'MS2_Spectra': [],
-                }
-            )
-        return new
+    def load_annotation_db(self, src='hmdb4'):
+        '''Database of known compound using JMS
+        '''
+        self.KCD = knownCompoundDatabase()
+        self.KCD.mass_indexed_compounds = pickle.load( pkg_resources.open_binary(db, 'mass_indexed_compounds.pickle') )
+        self.KCD.emp_cpds_trees = pickle.load( pkg_resources.open_binary(db, 'emp_cpds_trees.pickle') )
+
+
+    def db_mass_calibrate(self, required_calibrate_threshold=0.000002):
+        '''
+        Use KCD.evaluate_mass_accuracy_ratio to check systematic mass shift.
+        If greater than required_calibrate_threshold (default 2 ppm), 
+        calibrate m/z values for the whole experiment by updating self.CMAP.FeatureList.
+
+        good_reference_landmark_peaks: [{'ref_id_num': 99, 'apex': 211, 'height': 999999}, ...]
+        ref_id_num -> index number of mass track in MassGrid.
+        '''
+        mz_landmarks = [self.CMAP.MassGrid['mz'][p['ref_id_num']] for p in self.CMAP.good_reference_landmark_peaks]
+        mass_accuracy_ratio = self.KCD.evaluate_mass_accuracy_ratio(mz_landmarks, mode=self.mode, mz_tolerance_ppm=10)
+        if mass_accuracy_ratio:
+            if abs(mass_accuracy_ratio) > required_calibrate_threshold:
+                print("Mass shift is greater than %2.1f ppm. Correction applied." %(required_calibrate_threshold*1000000))
+                _correction = mass_accuracy_ratio + 1
+                for F in self.CMAP.FeatureList:
+                    F['mz'] = F['mz'] / _correction
+                    F['mz_corrected_by_division'] = _correction
+        else:
+            print("Mass accuracy check is skipped, too few mz_landmarks (%d) matched." %len(mz_landmarks))
+
+
+    def export_peak_annotation(self, dict_empCpds, KCD, export_file_name_prefix):
+        '''
+        interim_id is empCpd id. dict_empCpds example:
+        {'interim_id': 15,
+            'neutral_formula_mass': 100.112624453,
+            'neutral_formula': 'C6H14N',
+            'Database_referred': [],
+            'identity': [],
+            'MS1_pseudo_Spectra': [{'id_number': 'F117',
+            'mz': 100.11207049661286,
+            'apex': 221.0,
+            'ion_relation': 'anchor',
+            'parent_epd_id': 15},
+            {'id_number': 'F132',
+            'mz': 101.11543204162328,
+            'apex': 221.0,
+            'ion_relation': '13C/12C',
+            'parent_epd_id': 15}],
+            'MS2_Spectra': [],
+            'list_matches': [('C6H14N_100.112624', 'M[1+]', 2),
+            ('C6H13N_99.104799', 'M+H[1+]', 2)]}
+        {'interim_id': 1744,
+            'neutral_formula_mass': None,
+            'neutral_formula': None,
+            'Database_referred': [],
+            'identity': [],
+            'MS1_pseudo_Spectra': [{'id_number': 'F9723',
+                'mz': 131.11659958783318,
+                'apex': 427.0,
+                'ion_relation': 'anchor',
+                'parent_epd_id': 1744},
+            {'id_number': 'F6237',
+                'mz': 153.09836047808778,
+                'apex': 434.0,
+                'ion_relation': 'Na/H',
+                'parent_epd_id': 1744}],
+            'MS2_Spectra': []},
+        {'interim_id': 1745,
+            'neutral_formula_mass': 80.037448138,
+            'neutral_formula': 'C4H4N2',
+            'MS1_pseudo_Spectra': [{'id_number': 'F1',
+                'mz': 81.04470247776916,
+                'apex': 84.0,
+                'height': 172960.79114030077,
+                'cSelectivity': 1.0,
+                'goodness_fitting': 0.9466921897715592,
+                'snr': 35.0}]},...
+        '''
+        s = "[peak]id_number\tmz\tapex\t[EmpCpd]interim_id\t[EmpCpd]ion_relation\tneutral_formula\tneutral_formula_mass\tmatched_DB_shorts\tmatched_DB_records\n"
+        for ii, V in dict_empCpds.items():
+            matched_DB_shorts, matched_DB_records = '', ''
+            if 'list_matches' in V:
+                list_matches = V['list_matches']
+                #if list_matches:
+                matched_DB_shorts = ", ".join([ "(" + KCD.short_report_emp_cpd(xx[0]) + ")"  for xx in list_matches])
+                matched_DB_records = ", ".join([str(xx) for xx  in list_matches])
+
+            for peak in V['MS1_pseudo_Spectra']:
+                s += '\t'.join([str(x) for x in [
+                    peak['id_number'], peak['mz'], peak['apex'], V['interim_id'], peak.get('ion_relation', ''),
+                    V['neutral_formula'], V['neutral_formula_mass'],
+                    matched_DB_shorts, matched_DB_records]]) + "\n"
+
+        outfile = os.path.join(self.parameters['outdir'],export_file_name_prefix + '.tsv')
+        with open(outfile, 'w') as O:
+            O.write(s)
+
+        print("\nAnnotation of %d Empirical compounds was written to %s." %(len(dict_empCpds), outfile))
 
 
     def export_feature_table(self, full=True, outfile='cmap_feature_table.csv'):
@@ -188,6 +279,7 @@ class ext_Experiment(Experiment):
 
 
 
+
     def export_feature_table_old__(self, FeatureList, outfile='feature_table.tsv'):
         '''
         FeatureList: a list of namedTuples, i.e. Features; Output two files, one main, another low quality features.
@@ -223,9 +315,6 @@ class ext_Experiment(Experiment):
 
 
 
-
-
-
     #---------------------------------------------------------------------------------------------------------------
 
     def __obsolete__process_all(self):
@@ -316,6 +405,36 @@ class ext_Experiment(Experiment):
             O.write(s.encode('utf-8', 'ignore').decode('utf-8'))
 
         
+    def _reformat_epds_(self, list_empCpds, FeatureList):
+        '''
+        usage: list_empCpds = self._reformat_epds_(list_empCpds, self.CMAP.FeatureList)
+        '''
+        fDict = {}
+        for F in FeatureList:
+            fDict[F['id_number']] = F
+        new = []
+        for E in list_empCpds:
+            features = []
+            for peak in E['list_peaks']:
+                features.append(
+                    {'feature_id': peak[0], 
+                    'mz': fDict[peak[0]]['mz'], 
+                    'rtime': fDict[peak[0]]['apex'], 
+                    'charged_formula': '', 
+                    'ion_relation': peak[1]}
+                )
+            new.append(
+                {
+                'interim_id': E['id'], 
+                'neutral_formula_mass': None,
+                'neutral_formula': None,
+                'Database_referred': [],
+                'identity': [],
+                'MS1_pseudo_Spectra': features,
+                'MS2_Spectra': [],
+                }
+            )
+        return new
 
 
 
